@@ -6,8 +6,14 @@ from datup.io.dataio import (
     download_csv,
     download_csvm,
     upload_csv,
-    profiling_report,
+    #profiling_report,
     upload_model,
     upload_log,
-    download_excel
+    download_excel,
+    list_items,
+    delete_items,
+    get_model_name,
+    download_model
 )
+
+from datup.io.DatabasesIO import (DatabasesIO)
