@@ -2,7 +2,10 @@ import pandas as pd
 import s3fs
 import os
 import boto3
-from pandas_profiling import ProfileReport
+#from pandas_profiling import ProfileReport
+from statsmodels.tsa.statespace.exponential_smoothing import ExponentialSmoothingResults
+#from sklearn.externals import joblib
+import joblib
 
 def download_excel(
     self,
@@ -57,7 +60,9 @@ def download_csv(
     low_memory=False,
     indexcol=None,
     ts_csv = False,
-    freq=None
+    freq=None,
+    key_complete=False,
+    freq_offset=False
 ):
     r"""
     Return a dataframe downloaded from a specified datalake.
@@ -120,6 +125,15 @@ def download_csv(
             be different to None
         freq: str, default W-MON
             Is the frequency of getting data.
+        key_complete: bool, default False
+            If key_complete is True then the stage path of element in s3 bucket must be the full path without s3 prefix
+            and datalake.
+        freq_offset: bool, default False
+            The freq_offset parameter defines the time frequency which dataframe would be downloaded. If True, then
+            the function will execute the following command
+            * df.index.freq=pd.tseries.frequencies.to_offset(freq)
+            If false then
+            * df.index.freq=freq
 
     :rtype: DataFrame
     :return: A DataFrame is return as two-dimensional data structure
@@ -131,31 +145,64 @@ def download_csv(
     """
 
     try:
-        complete_path = str(os.path.join(self.prefix_s3, self.datalake,stage,filename+".csv")).replace("\\","/")
-        if ts_csv and indexcol is not None and freq is not None:
-            df = pd.read_csv(
-                self.aws_credentials_s3fs.open(complete_path),
-                sep=sep, 
-                encoding=encoding,
-                infer_datetime_format=infer_datetime_format,
-                low_memory=low_memory, 
-                index_col=indexcol
-            )
-            df.index = pd.to_datetime(df.index)
-            df.index.freq=freq
-        elif ts_csv and indexcol is None and freq is not None:
-            print("Is necessary an indexcol value")
-        elif ts_csv and indexcol is not None and freq is None:
-            print("Is necessary a freq value")            
-        else:                
-            df = pd.read_csv(
-                self.aws_credentials_s3fs.open(complete_path),
-                sep = sep,
-                encoding= encoding, 
-                infer_datetime_format = infer_datetime_format,
-                low_memory=low_memory, 
-                parse_dates=datecols
-            )        
+        if key_complete:
+            complete_path = str(os.path.join(self.prefix_s3,self.datalake,stage)).replace("\\","/")
+            if ts_csv and indexcol is not None and freq is not None:
+                df = pd.read_csv(
+                    self.aws_credentials_s3fs.open(complete_path),
+                    sep=sep, 
+                    encoding=encoding,
+                    infer_datetime_format=infer_datetime_format,
+                    low_memory=low_memory, 
+                    index_col=indexcol
+                )
+                df.index = pd.to_datetime(df.index)
+                if freq_offset:
+                    df.index.freq=pd.tseries.frequencies.to_offset(freq)
+                else: 
+                    df.index.freq=freq
+            elif ts_csv and indexcol is None and freq is not None:
+                print("Is necessary an indexcol value")
+            elif ts_csv and indexcol is not None and freq is None:
+                print("Is necessary a freq value")            
+            else:                
+                df = pd.read_csv(
+                    self.aws_credentials_s3fs.open(complete_path),
+                    sep = sep,
+                    encoding= encoding, 
+                    infer_datetime_format = infer_datetime_format,
+                    low_memory=low_memory, 
+                    parse_dates=datecols
+                )     
+        else:
+            complete_path = str(os.path.join(self.prefix_s3, self.datalake,stage,filename+".csv")).replace("\\","/")
+            if ts_csv and indexcol is not None and freq is not None:
+                df = pd.read_csv(
+                    self.aws_credentials_s3fs.open(complete_path),
+                    sep=sep, 
+                    encoding=encoding,
+                    infer_datetime_format=infer_datetime_format,
+                    low_memory=low_memory, 
+                    index_col=indexcol
+                )
+                df.index = pd.to_datetime(df.index)
+                if freq_offset:
+                    df.index.freq=pd.tseries.frequencies.to_offset(freq)
+                else: 
+                    df.index.freq=freq
+            elif ts_csv and indexcol is None and freq is not None:
+                print("Is necessary an indexcol value")
+            elif ts_csv and indexcol is not None and freq is None:
+                print("Is necessary a freq value")            
+            else:                
+                df = pd.read_csv(
+                    self.aws_credentials_s3fs.open(complete_path),
+                    sep = sep,
+                    encoding= encoding, 
+                    infer_datetime_format = infer_datetime_format,
+                    low_memory=low_memory, 
+                    parse_dates=datecols
+                )        
     except FileNotFoundError as error:
         self.logger.exception(f"Exception found: {error}") 
         raise
@@ -227,7 +274,8 @@ def upload_csv(
     index=False,
     header=True,
     date_format="%Y-%m-%d",
-    ts_csv=False
+    ts_csv=False,
+    encoding= 'UTF-8'
 ):  
     r"""
     Return a uri where the dataframes was uploaded in csv format.
@@ -259,6 +307,9 @@ def upload_csv(
             Format string for datetime objects.
         ts_csv: bool, default False
             If ts_csv is True then activates the ts_csv upload. If True, index must be different to False
+        encoding: str, default ‘utf-8’
+            Took it from Pandas to_csv encoding description
+            Encoding to use for UTF when reading/writing (ex. ‘utf-8’).
         
     :rtype: Str
     :return: The uri where DataFrame was uploaded into S3
@@ -274,14 +325,14 @@ def upload_csv(
         complete_local_path = str(os.path.join(self.local_path,filename+".csv")).replace("\\","/")
         medium_s3_path = str(os.path.join(stage, filename, filename+'.csv').replace("\\","/"))
         if ts_csv and index:
-            df.to_csv(complete_local_path, index=index, header=header, date_format=date_format)
+            df.to_csv(complete_local_path, index=index, header=header, date_format=date_format, encoding = encoding)
             self.aws_credentials_s3.Bucket(self.datalake).upload_file(complete_local_path, medium_s3_path)
         if ts_csv and not index:
             print("Is necessary use index True for ts_csv mode")
         if index and not ts_csv:
             print("Is necessary use ts_csv True")
         else:
-            df.to_csv(complete_local_path, index=index, header=header, date_format=date_format)
+            df.to_csv(complete_local_path, index=index, header=header, date_format=date_format, encoding = encoding)
             self.aws_credentials_s3.Bucket(self.datalake).upload_file(complete_local_path, medium_s3_path)                
         upload_path = complete_s3_path
     except FileNotFoundError as error:
@@ -289,73 +340,74 @@ def upload_csv(
         raise    
     return upload_path
 
-def profiling_report(
-    self,
-    df,
-    stage=None,
-    filename=None,
-    save=False,
-    minimal=True
-):
-    r"""
-    Return a html embbeded with the DataFrame profiling data or save it into a S3 datalake .
-    This function takes the aws credentials from DataIO class and use it for upload
-    the required data, after this, the method can save or not the result by the save flag.
-
-    THIS METHOD HAS BE TESTED
-
-    Parameters
-    ----------
-        self: Datup
-            It is the object type Datup necessary to upload using aws credentials and write the log. The 
-            default folder for log is /tmp/
-        df: DataFrame
-            Is the DataFrame for profiling the data
-        stage: str, default None
-            It is the set of folders after the datalake to the file that is required to upload,
-            this method just apply if save parameter is True 
-        filename: str, default None
-            Is the name of the filename to upload without html suffix, 
-            this method just apply if save parameter is True
-        save: bool, default False
-            If save is False, the html was embedded in the worksheet if True, the html will be saved into a 
-            S3 datalake
-        minimal: bool, default True
-            If minimal is True the method disables expensive computations (such as correlations and dynamic binning)
-        
-    :rtype: Html
-    :return: A Html is returned by the method
-                
-    Examples
-    --------
-    >>> import datup as dt
-    >>> dt.generate_profiling(object,df)        
-    """
-
-    try:
-        profile = ProfileReport(df, minimal=minimal)
-        if save: 
-            complete_s3_path = str(os.path.join(self.prefix_s3, self.datalake,stage,filename,"{}.html".format(filename))).replace("\\","/")         
-            medium_s3_path = str(os.path.join(stage, filename,"{}.html".format(filename))).replace("\\","/")
-            profile.to_file("{}{}.html".format(self.local_path,filename))
-            self.aws_credentials_s3.Bucket(self.datalake).upload_file("{}{}.html".format(self.local_path,filename), medium_s3_path)
-            return complete_s3_path
-        else:
-            return profile.to_notebook_iframe()
-    except:
-        self.logger.exception(f"Exception found: ")  
-        raise
+#def profiling_report(
+#    self,
+#    df,
+#    stage=None,
+#    filename=None,
+#    save=False,
+#    minimal=True
+#):
+#    r"""
+#    Return a html embbeded with the DataFrame profiling data or save it into a S3 datalake .
+#    This function takes the aws credentials from DataIO class and use it for upload
+#   the required data, after this, the method can save or not the result by the save flag.#
+#    THIS METHOD HAS BE TESTED
+#
+#    Parameters
+#    ----------
+#        self: Datup
+#            It is the object type Datup necessary to upload using aws credentials and write the log. The 
+#            default folder for log is /tmp/
+#        df: DataFrame
+#            Is the DataFrame for profiling the data
+#        stage: str, default None
+#            It is the set of folders after the datalake to the file that is required to upload,
+#            this method just apply if save parameter is True 
+#        filename: str, default None
+#            Is the name of the filename to upload without html suffix, 
+#            this method just apply if save parameter is True
+#        save: bool, default False
+#            If save is False, the html was embedded in the worksheet if True, the html will be saved into a 
+#            S3 datalake
+#       minimal: bool, default True
+#           If minimal is True the method disables expensive computations (such as correlations and dynamic binning)
+#        
+#    :rtype: Html
+#    :return: A Html is returned by the method
+#                
+#    Examples
+#    --------
+#    >>> import datup as dt
+#    >>> dt.generate_profiling(object,df)        
+#    """
+#
+#    try:
+#        profile = ProfileReport(df, minimal=minimal)
+#        if save: 
+#            complete_s3_path = str(os.path.join(self.prefix_s3, self.datalake,stage,filename,"{}.html".format(filename))).replace("\\","/")         
+#            medium_s3_path = str(os.path.join(stage, filename,"{}.html".format(filename))).replace("\\","/")
+#            profile.to_file("{}{}.html".format(self.local_path,filename))
+#            self.aws_credentials_s3.Bucket(self.datalake).upload_file("{}{}.html".format(self.local_path,filename), medium_s3_path)
+#            return complete_s3_path
+#        else:
+#            return profile.to_notebook_iframe()
+#    except:
+#        self.logger.exception(f"Exception found: ")  
+#        raise
     
 def upload_model(
     self,
     model,
     stage=None,
-    filename=None
+    item=None,
+    filename=None,
+    gridsearch=False
 ):
     r"""
     The upload_model save a pkl model and return the S3 datalake path where saved.
 
-    THIS METHOD HASN'T BE TESTED
+    THIS METHOD HAS BE TESTED
 
     Parameters
     ----------
@@ -368,20 +420,28 @@ def upload_model(
             It is the set of folders after the datalake to the file that is required to upload,
         filename: str, default None
             Is the name of the filename to upload without pkl suffix, 
-                
+        item: str, default None
+            Is the name of item for a model. This method is used by upload_best_model in ExponetialSmoothing class
+        gridsearch: bool, default False
+            If gridsearch is True then a sklearn will be saved
+
     :rtype: Str
     :return: The path where model saved is returned
                 
     Examples
     --------
     >>> import datup as dt
-    >>> dt.upload_model(object,model,stage="stage",filename="filename")        
+    >>> dt.upload_model(object,model,stage="stage",item="item_id",filename="filename")        
     """
 
-    try:        
-        complete_s3_path = str(os.path.join(self.prefix_s3, self.datalake, stage, filename, "{}.pkl".format(filename))).replace("\\","/")
+    try:  
+        complete_s3_path = str(os.path.join(self.prefix_s3, self.datalake, stage, item, "{}.pkl".format(filename))).replace("\\","/")
         route_model = str(os.path.join(self.local_path,"{}.pkl".format(filename))).replace("\\","/") 
-        medium_s3_path = str(os.path.join(stage, filename, "{}.pkl".format(filename))).replace("\\","/")
+        medium_s3_path = str(os.path.join(stage, item, "{}.pkl".format(filename))).replace("\\","/")     
+        if gridsearch:
+          joblib.dump(model,str(os.path.join(self.local_path,"{}.pkl".format(filename)).replace("\\","/")))
+        else:          
+          model.save(route_model)
         self.aws_credentials_s3.Bucket(self.datalake).upload_file(route_model, medium_s3_path)
     except FileNotFoundError as error:
         self.logger.exception(f"Exception found: {error}")  
@@ -396,7 +456,7 @@ def upload_log(
     r"""
     The upload_log save the log of process and return the S3 datalake path where saved.
 
-    THIS METHOD HASN'T BE TESTED
+    THIS METHOD HAS BE TESTED
 
     Parameters
     ----------
@@ -406,7 +466,7 @@ def upload_log(
         logfile: str
             Is the log model
         stage: str, default None
-            It is the set of folders after the datalake to the file that is required to upload,
+            It is the set of folders after the datalake to the file that is required to upload
                 
     :rtype: Str
     :return: The path where log saved is returned
@@ -427,4 +487,160 @@ def upload_log(
         raise    
     return complete_s3_path
 
+def list_items(
+    self,
+    stage
+):
+    """
+    Return list of items files stored in the datalake
+    
+    THIS METHOD HAS BE TESTED
+
+    Parameters
+    ----------
+        self: Datup
+            It is the object type Datup necessary to upload using aws credentials and write the log. The 
+            default folder for log is /tmp/
+        stage: str
+            It is the set of folders after the datalake to the files that is required to download
+                
+    :rtype: List
+    :return: The list of items queried in the dataframe
+                
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.list_items(object,stage="stage") 
+    """
+    try: 
+        items = [e['Key'][:] for p in self.aws_client_s3.get_paginator('list_objects_v2').paginate(Bucket=self.datalake, Prefix=stage) for e in p['Contents']]
+    except IOError as error:
+        self.logger.exception(f"Exception found: {error}") 
+        raise
+    return items
+
+def delete_items(
+    self,
+    stage,
+    item_id
+):
+    """
+    Return a boolean flag indicating success for items deletion in the datalake
+    
+    THIS METHOD HAS BE TESTED
+
+    Parameters
+    ----------
+        self: Datup
+            It is the object type Datup necessary to upload using aws credentials and write the log. The 
+            default folder for log is /tmp/
+        stage: str,
+            It is the set of folders after the datalake to the files that is required to download
+        item_id: str,
+            Is the all path of item to delete    
+                
+    :rtype: Bool
+    :return: A True or False flag indicating success for deletion
+                
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.delete_items(object,stage="stage",item_id="item_id")    
+    """
+    try:
+        for it in self.aws_client_s3.get_paginator('list_objects_v2').paginate(Bucket=self.datalake, Prefix=os.path.join(stage, item_id).replace("\\","/")):
+            key_counts = it.get('KeyCount')
+        if key_counts > 0:
+            keys = [e['Key'][:] for p in self.aws_client_s3.get_paginator('list_objects_v2').paginate(Bucket=self.datalake, Prefix=os.path.join(stage, item_id).replace("\\","/")) for e in p['Contents'][-1:]]
+            for key in keys:            
+                kwargs = {'Bucket':self.datalake, 'Key':key}
+                response = self.aws_client_s3.delete_object(**kwargs)
+                response = response.get('DeleteMarker')
+        else: 
+            response = True
+    except IOError as error:
+        self.logger.exception(f"Exception found: {error}") 
+        raise
+    return response
+
+def get_model_name(
+    self,
+    stage,
+    item_id
+):
+    """
+    Return list of models' names stored in the datalake
+    
+    THIS METHOD HAS BE TESTED
+
+    Parameters
+    ----------
+        self: Datup
+            It is the object type Datup necessary to upload using aws credentials and write the log. The 
+            default folder for log is /tmp/
+        stage: str,
+            It is the set of folders after the datalake to the files that is required to download
+        item_id: str,
+            Is the all path of item to delete    
+                
+    :rtype: List
+    :return: A list of model names per items
+                
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.get_model_name(object,stage="stage",item_id="item_id") 
+    """
+    try: 
+        item = [e['Key'][-10:-4] for p in self.aws_client_s3.get_paginator('list_objects_v2').paginate(Bucket=self.datalake, Prefix=os.path.join(stage, item_id)) for e in p['Contents']]
+    except IOError as error:
+        self.logger.exception(f"Exception found: {error}")
+        raise
+    return item
+
+def download_model(
+    self,
+    stage,
+    item_id,
+    filename,
+    gridsearch=False
+):
+    """
+    Return a pkl model from a S3 Bucket
+    
+    THIS METHOD HAS BE TESTED
+
+    Parameters
+    ----------
+        self: Datup
+            It is the object type Datup necessary to upload using aws credentials and write the log. The 
+            default folder for log is /tmp/
+        stage: str,
+            It is the set of folders after the datalake to the files that is required to download
+        item_id: str,
+            Is the all path of item to delete    
+        filename: str,
+            Is the name of file inside of the item_id path which contains the pkl model
+                
+    :rtype: Object
+    :return: The ExponentialSmoothingResults object
+                
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.download_model(object,stage="stage",item_id="item_id",filename="filename")    
+    """
+    try:
+        file_pickle = os.path.join(self.local_path, '{}.pkl'.format(filename)).replace("\\","/")
+        s3_route_pickle = os.path.join(stage, item_id, '{}.pkl'.format(filename)).replace("\\","/")
+        self.aws_credentials_s3.Bucket(self.datalake).download_file(s3_route_pickle, file_pickle)
+        if gridsearch:          
+          model = joblib.load(file_pickle)
+        else:
+          model = ExponentialSmoothingResults.load(file_pickle)
+    except FileNotFoundError as error:
+        self.logger.exception(f"Exception found: {error}")
+        raise
+    return model
+
 
