@@ -246,6 +246,118 @@ def profile_dataset(self,df):
         raise 
     return df_profile
 
+def rank_sku_abc(
+    df,
+    sort_dim,
+    abc_thres_list=[0.8, 0.95, 1.1]
+):
+    """
+    Return a dataframe ranking the SKUs in ABC classes based on the  sorting dimension
+
+    Parameters
+    __________
+    df: DataFrame
+        It is the DataFrame with which want to work
+    sort_dim: str
+        Is the dataframe's dimension to classify the SKUs (e.g. quantity, costs, revenues)
+    abc_thers_list: list
+        Is the list of thresholds to classify SKU into A, B or C
+
+    :return: A DataFrame with SKUs ranked as ABC
+    :rtype: DataFrame
+
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.rank_sku_abc(df,sort_dim,abc_thres_list)
+    """
+    try:
+        df_abc = df.sort_values([sort_dim+' sum'], ascending=False).reset_index(drop=True)
+        df_abc[sort_dim+' sum Pct'] = df_abc[sort_dim+' sum']/df_abc[sort_dim+' sum'].sum()
+        df_abc[sort_dim+' sum Pct Acum'] = df_abc[sort_dim+' sum Pct'].cumsum()
+        df_abc.loc[df_abc[sort_dim+' sum Pct Acum']<=abc_thres_list[0],'ABC'] = 'A'
+        df_abc.loc[(df_abc[sort_dim+' sum Pct Acum']>abc_thres_list[0]) & (df_abc[sort_dim+' sum Pct Acum']<=abc_thres_list[1]),'ABC'] = 'B'
+        df_abc.loc[(df_abc[sort_dim+' sum Pct Acum']>abc_thres_list[1]) & (df_abc[sort_dim+' sum Pct Acum']<=abc_thres_list[2]),'ABC'] = 'C'
+    except IOError as error:
+        print(f'Exception found: {error}')
+        raise
+    return df_abc
+
+def rank_sku_xyz(
+    df,
+    sort_dim,
+    xyz_thres_list=[0.33, 0.66]
+):
+    """
+    Return a dataframe ranking the SKUs in XYZ classes based on the  sorting dimension
+
+    Parameters
+    __________
+    df:
+        It is the DataFrame with which want to work
+    sort_dim:
+        Is the dataframe's dimension to classify the SKUs (e.g. quantity, costs, revenues)
+    xyz_thres_list:
+        Is the list of thresholds to classify SKU into X, Y or Z
+
+    :return: A DataFrame with SKUs ranked as XYZ
+    :rtype: DataFrame
+
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.rank_sku_xyz(df,sort_dim,xyz_thres_list)
+    """
+    try:
+        xyz_list = []
+        df[sort_dim+' std Pct'] = (df[sort_dim+' std']/df[sort_dim+' mean']).fillna(0)
+        for val in df[sort_dim+' std Pct'].values:
+            if val <= xyz_thres_list[0]: xyz_list.append('X')
+            elif val > xyz_thres_list[0] and val <= xyz_thres_list[1]: xyz_list.append('Y')
+            else: xyz_list.append('Z')
+        df['XYZ'] = xyz_list
+    except IOError as error:
+        print(f'Exception found: {error}')
+        raise
+    return df
+
+def rank_sku_fsn(
+    df,
+    sort_dim,
+    fsn_thres_list=[0.5, 0.75]
+):
+    """
+    Return a dataframe ranking the SKUs in FSN classes based on the sorting dimension
+
+    Parameters
+    __________
+    df:
+        It is the DataFrame with which want to work
+    sort_dim:
+        Is the dataframe's dimension to classify the SKUs (e.g. quantity, costs, revenues)
+    fsn_thres_list:
+        Is the list of thresholds to classify SKU into F, S or N
+
+    :return: A DataFrame with SKUs ranked as FSN
+    :rtype: DataFrame
+
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.rank_sku_fsn(df,sort_dim,fsn_thres_list)
+    """
+    try:
+        fsn_list = []
+        for val in df[sort_dim+' run'].values:
+            if val <= fsn_thres_list[0]: fsn_list.append('N')
+            elif val > fsn_thres_list[0] and val <= fsn_thres_list[1]: fsn_list.append('S')
+            else: fsn_list.append('F')
+        df['FSN'] = fsn_list
+    except IOError as error:
+        print(f'Exception found: {error}')
+        raise
+    return df
+
 def replace_categorical_na(
     self,
     df,
@@ -324,4 +436,37 @@ def replace_numeric_na(
     except IOError as error:
         self.logger.exception(f'Exception found: {error}')
         raise 
+    return df
+
+def replace_datetime_na(self,df):
+    """
+    Returns a dataframe without na values in datetime dimensions
+    
+    THIS METHOD HAS BE TESTED
+    
+    Parameters
+    ----------
+        self: Datup
+            It is the object type Datup necessary to download using aws credentials and write the log. The 
+            default folder for log is /tmp/
+        df: DataFrame
+            It is the DataFrame with which want to work
+
+    :return: A DataFrame without na values in datetime dimensions 
+    :rtype: DataFrame
+
+    Examples
+    --------
+    >>> import datup as dt
+    >>> dt.replace_datetime_na(object,df,na_values=0)
+    """
+    try:
+        datetime_dims = [dim for dim in df.columns if df[dim].dtype=='datetime64']
+        for dim in datetime_dims:
+            df[dim] = pd.to_datetime(df[dim], errors='coerce')    
+            na_flag = df[dim].isnull().values.any()
+            self.logger.debug('{dimension} has NaNs: {flag}'.format(dimension=dim, flag=na_flag))  
+    except IOError as error:
+        self.logger.exception(f'Exception found: {error}')
+        raise 
     return df
\ No newline at end of file
